#!/bin/bash
if [ \( "$#" -eq 0 \) ] ; then
	echo "usage: ${0} <experiment-name>"
	exit 1
fi

EXPERIMENT="$1"
LOG_FILE="tacas18/${EXPERIMENT}.log"
LABELS_FILE="tacas18/${EXPERIMENT}-labels.log"

AUT="$(cat ${LOG_FILE} | grep 'NFA:' | sed 's/.*\/\(.*reduced.*\),.*/\1/' | head -n 1)"
MODE="$(cat ${LOG_FILE} | grep 'Mode:' | sed 's/^Mode: \([^,]*\),.*$/\1/' | head -n 1)"
ORIG_SIZE="$(cat ${LOG_FILE} | grep '^#States' | sed 's/^#States (original): \([^,]*\),.*$/\1/' | head -n 1)"
TIME_LABEL="$(cat ${LABELS_FILE} | grep '^real' | sed 's/^real\t\(.*\)s$/\1/' | head -n 1)"

echo "Results for ${AUT}, mode ${MODE}, size of original: ${ORIG_SIZE}, time labels: ${TIME_LABEL}"

gawk '\
	function out_line() \
		{ \
			printf N; \
			printf ";" size_reduced; \
			printf ";" size_rabitted; \
			printf ";" error_bound; \
			printf ";" exact_error; \
			printf ";" time_app; \
			printf ";" time_rabit; \
			printf ";" time_exact; \
			print ""; \
		} \
	BEGIN \
		{ \
			first = 1; \
			stage = 0; \
			\
			printf("N;"); \
			printf("size app;"); \
			printf("size app+rabit;"); \
			printf("error bound;"); \
			printf("exact error;"); \
			printf("time app;"); \
			printf("time rabit;"); \
			printf("time exact"); \
			print "";
		} \
	/^NFA:/ \
		{ \
			if (first != 1) { \
				out_line();
			} \
			first = 0; \

			stage = 0; \
			N = "N/A"; \
			size_reduced = "N/A"; \
			size_rabitted = "N/A"; \
			error_bound = "N/A"; \
			exact_error = "N/A"; \
			time_app = "N/A"; \
			time_rabit = "N/A"; \
			time_exact = "N/A"; \
		} \
	/Restriction: / \
		{ \
			aux = gensub(/^.*Restriction: ([^,]*),.*$/, "\\1", "1"); \
			N = aux; \
		} \
	/Input automaton:/ \
		{ \
			aux = gensub(/^.*\# of States ([^.]*)\.*$/, "\\1", "1"); \
			size_reduced = aux \
		} \
	/Reduced automaton:/ \
		{ \
			aux = gensub(/^.*\# of States ([^.]*)\.*$/, "\\1", "1"); \
			size_rabitted = aux \
		} \
	/^The distance upperbound/ \
		{ \
			aux = gensub(/^.*upperbound is (.*)$/, "\\1", "1"); \
			error_bound = aux \
		} \
	/^The probabilistic distance of input/ \
		{ \
			aux = gensub(/^.*automata is (.*)\..*$/, "\\1", "1"); \
			exact_error = aux \
		} \
	/^real/ \
		{ \
			aux = gensub(/^real\t(.*)s$/, "\\1", "1"); \
			if (stage == 0) {\
				stage = 1; \
				time_app = aux \
			} \
			else if (stage == 1) {\
				stage = 2; \
				time_rabit = aux \
			} \
			else if (stage == 2) { \
				stage = 0; \
				time_exact = aux \
			} \
		} \
	END \
		{ \
			out_line();
		} \
	' ${LOG_FILE} | column -s ';' -t
